﻿package
{
	import flash.geom.Point;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.events.TimerEvent;
	import flash.display.MovieClip;
	
	public class LetterGrid extends MovieClip
	{
		// Public variables
		// ------------------------------------------------------------------------------------------- //
		public const CELL_SPACE_WIDTH :Number = 29;
		public const CELL_SPACE_HEIGHT:Number = 29;
		
		public const LETTER_GRID_WIDTH :Number = 16;
		public const LETTER_GRID_HEIGHT:Number = 16;
		
		// Private variables
		// ------------------------------------------------------------------------------------------- //
			// Token for unused cell. Anything not between A-Z will work
		private const TOKEN:String = "*";
			// Words and letters arrays
		private var words:Array = new Array();
		private var letters:Array = new Array(LETTER_GRID_HEIGHT);
			// Current word marker
		private var wordMarker:WordMarker;
		
		public function LetterGrid()//(wordList:Array)
		{			
				// Populate with tokens
			for(var y:int=0; y<LETTER_GRID_HEIGHT; ++y) {
					// 2D array
				letters[y] = new Array(LETTER_GRID_WIDTH);
				for(var x:int=0; x<LETTER_GRID_WIDTH; ++x) {
						// Create a grid full of tokens
					var letter:Letter = new Letter(TOKEN);
					
						// Position letter at center of cell
					letter.x = -(this.width  / 2) + (CELL_SPACE_WIDTH  * (x + 0.5));
					letter.y = -(this.height / 2) + (CELL_SPACE_HEIGHT * (y + 0.5));
						
						// Add the letter
					addChild(letter);
						// Save a refrence on the grid
					letters[y][x] = letter;
				}
			}
			
				// Add mouse listeners for word selection
			addEventListener(MouseEvent.MOUSE_DOWN, selectionStart);
			addEventListener(MouseEvent.MOUSE_OVER, selectionPrograss);
			addEventListener(MouseEvent.MOUSE_UP  , selectionEnd);
			addEventListener(MouseEvent.ROLL_OUT  , selectionEnd);
			
				// Populate when starting
			addEventListener(Event.ADDED_TO_STAGE, function(event:Event) {
				populate();
				
					// Add time event, broadcasted to root
				root.addEventListener(TimerEvent.TIMER_COMPLETE, function(event:TimerEvent) {
						// Player lost :(
					finish(false);
				});
			});
			
				// Reset when removed
			addEventListener(Event.REMOVED_FROM_STAGE, function(event:Event) {
				reset();
				
					// Regenerate words for next time
				ResourceManager.randomize();
			});
		}
		
		public function reset()
		{
				// Set the letters to tokens again
			for(var y:int=0; y<LETTER_GRID_HEIGHT; ++y) {
				for(var x:int=0; x<LETTER_GRID_WIDTH; ++x)
					letters[y][x].text = TOKEN;
			}
			
				// Remove all markers
			for(var i:int=0; i<numChildren; ++i) {
				if(getChildAt(i) is WordMarker)
					removeChildAt(i--);
			}
			
				// Populate again
			populate();
		}
		
		// Private methods
		// ------------------------------------------------------------------------------------------- //
		private function populate()
		{
				// Retreieve words, convert all words to upper case and filter
				// out duplicate words as well as clean all spaces from the words
			words = ResourceManager.words.map(function(item:*) {
				return item.split(" ").join("").toUpperCase();
			}).filter(function(item:*, index:int, array:Array):Boolean {
				return array.indexOf(item, index + 1) == -1;
			});
			
				// Keep a copy as we elimiate what was already placed
			var placementWords:Array = words.concat();
			
				// Start placing the words
			placeWord:while(placementWords.length > 0) {
					// Random Position
				var px:int = Math.floor(Math.random() * LETTER_GRID_WIDTH );
				var py:int = Math.floor(Math.random() * LETTER_GRID_HEIGHT);
					
					// Cache booleans as numbers (1 or 0)
				var horizontal:Number = Number(ResourceManager.config.horizontal);
				var vertical  :Number = Number(ResourceManager.config.vertical  );
				var diagonal  :Number = Number(ResourceManager.config.diagonal  );
					
					// Random Direction, respecting config file
				var randDir:int = Math.floor(Math.random() * (2 + diagonal)) + 1;
					// Some bit twiddling won't hurt - using a 1.1 bitfield as direction delta
				var dx:int = (randDir >> 0) & horizontal;
				var dy:int = (randDir >> 1) & vertical  ;
				
					// Don't allow both to be 0
				if((dx | dy) == 0) {
					dx = horizontal | diagonal;
					dy = vertical   | diagonal;
				}
				
					// If reverse is allowed, randomly reverse a direction
				if(ResourceManager.config.reverse) {
					dx *= (Math.floor(Math.random() * 2) - 1) | 1;
					dy *= (Math.floor(Math.random() * 2) - 1) | 1;
				}
					
					// Retrieve the word
				var word:String = placementWords[0];
					
					// Check each block to see if word fits. Restart if it doesn't
				for(var i=0; i<word.length; ++i) {
						// Can't underflow
					if((px + (dx*i) < 0) || (py + (dy*i) < 0))
						continue placeWord;
						// Can't overflow
					if((px + (dx*i) >= LETTER_GRID_WIDTH) || (py + (dy*i) >= LETTER_GRID_HEIGHT))
						continue placeWord;
					
						// Already taken
					var curChar:String = letters[py + (dy*i)][px + (dx*i)];
					if((curChar != TOKEN) && (curChar != word.toUpperCase().charAt(i)))
						continue placeWord;
				}
				
					// If we got here, word can be saftly placed
				for(i=0; i<word.length; ++i)
					letters[py + dy*i][px + dx*i].text = word.toUpperCase().charAt(i);
					
					// Remove the word
				placementWords.shift();
			}
			
				// Replace tokens with random letter
			for(var y:int=0; y<LETTER_GRID_HEIGHT; ++y) {
				for(var x:int=0; x<LETTER_GRID_WIDTH; ++x) {
					if(letters[y][x] == TOKEN)
						letters[y][x].text = String.fromCharCode(Math.floor((Math.random() * (0x5A - 0x41)) + 0x41));
				}
			}
		}
		
		private function selectionStart(event:MouseEvent)
		{
				// Create a marker
			wordMarker = new WordMarker(this);
			
				// Set it to the current cell
			wordMarker.start = new Point(Math.floor(mouseX / CELL_SPACE_WIDTH), Math.floor(    mouseY / CELL_SPACE_HEIGHT));
			
				// Add to display list
			addChild(wordMarker);
		}
		
		private function selectionPrograss(event:MouseEvent)
		{
				// No selection
			if(wordMarker == null)
				return;
				
				// No selection
			if(!contains(wordMarker))
				return;
			
				// Set end
			wordMarker.end = new Point(Math.floor(mouseX / CELL_SPACE_WIDTH), Math.floor(mouseY / CELL_SPACE_HEIGHT));
		}
		
		private function selectionEnd(event:MouseEvent)
		{
				// No selection
			if(wordMarker == null)
				return;
				
				// No selection
			if(!contains(wordMarker))
				return;
				
				// Get marked word and it's reverse
			var word:String = retrieveWord(wordMarker);
			var reverseWord:String = word.split("").reverse().join("");
				
				// Check reverse first and save it as normal
			if(words.indexOf(reverseWord) != -1)
				word = reverseWord;

				// Check if word is in our bank
			if(words.indexOf(word) != -1) {
					// This will not return -1 since we already know the word is in the array
				words.splice(words.indexOf(word), 1);

					// Create a new marker
				wordMarker = new WordMarker(this);
				
					// Transmit 'word found' event using root
				root.dispatchEvent(new WordEvent(WordEvent.FOUND, word));
				
					// Go to victory frame if done
				if(words.length == 0)
					finish(true);
			} else
				removeChild(wordMarker);
		}
		
		private function retrieveWord(marker:WordMarker):String
		{
				// Retrieve start and end points
			var start:Point = marker.start;
			var end  :Point = marker.end  ;
				
				// Convert between the two systems
			start.offset(LETTER_GRID_WIDTH / 2, LETTER_GRID_HEIGHT / 2);
			end  .offset(LETTER_GRID_WIDTH / 2, LETTER_GRID_HEIGHT / 2);
				
				// Compute delta
			var delta:Point = new Point(end.x - start.x, end.y - start.y);
			var length:int = Math.max(Math.abs(delta.x), Math.abs(delta.y)) + 1;

				// Compute directions
			if(delta.x != 0)
				delta.x /= Math.abs(delta.x);
			if(delta.y != 0)
				delta.y /= Math.abs(delta.y);
			
				// Generate word from the grid
			var word:String = new String();
			for(var i:int=0; i<length; ++i)
				word += letters[start.y + (delta.y * i)][start.x + (delta.x * i)];
			
			return word;
		}
		
		private function finish(victory:Boolean)
		{
				// "Beautifulify"
			var base:MovieClip = root as MovieClip;
				
				// Go to the appropriate frame
			base.gotoAndStop(base.currentFrame + victory);
		}
	}
}
