﻿package
{
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.display.Sprite;
	
	public class WordMarker extends Sprite
	{
			// Ref to 'parent' letter grid for dimentions
		private var letterGrid:LetterGrid;
		var limitRect:Rectangle;
		
			// Start and end points
		private var startPoint:Point = new Point(-1, -1);
		private var endPoint  :Point = new Point(-1, -1);
		
		public function WordMarker(_letterGrid:LetterGrid)
		{
			letterGrid = _letterGrid;
				
				// Create a virtual rectangle for limitation
			limitRect = new Rectangle(-letterGrid.LETTER_GRID_WIDTH  / 2,
									  -letterGrid.LETTER_GRID_HEIGHT / 2,
									   letterGrid.LETTER_GRID_WIDTH,
									   letterGrid.LETTER_GRID_HEIGHT);
		}
		
		public function get start():Point 
		{
				// Clone to prevent reference trashing
			return startPoint.clone();
		}
		
		public function set start(point:Point) 
		{
				// Save start point
			startPoint = limitPoint(point);
			
				// Draw a 1x1 selection
			end = limitPoint(point);
		}
		
		public function get end():Point 
		{
				// Clone to prevent reference trashing
			return endPoint.clone();
		}
		
		public function set end(point:Point) 
		{
				// Save end point
			endPoint = limitPoint(point);
						
				// Delta
			var delta:Point = endPoint.subtract(startPoint);
						
				// Only allow horizonral, vertical or diagonal selections
			if(Math.abs(delta.x) != Math.abs(delta.y) && !delta.x == !delta.y) {
				var rectN:Number = Math.max(Math.abs(delta.x), Math.abs(delta.y));
				var length:Point = new Point(rectN * (1 | -int(delta.x < 0)), rectN * (1 | -int(delta.y < 0)));
				
					// 3 Possible points
				var horizontal:Point = new Point(point.x, start.y);
				var vertical  :Point = new Point(start.x, start.y + length.y);
				var diagonal  :Point = new Point(start.x + length.x, start.y + length.y);
				
					// Limit all points in their own direction
					// Limit horizonal point
				if(outside(horizontal)) {
					if(horizontal.x < limitRect.x)
						horizontal.x = limitRect.x;
					else
						horizontal.x = limitRect.right;
				}
					// Limit vertical point					
				if(outside(vertical)) {
					if(vertical.y < limitRect.y)
						vertical.y = limitRect.y;
					else
						vertical.y = limitRect.bottom;
				}
					// Limit diagonal point
				length.x /= -rectN;
				length.y /= -rectN;
				while(outside(diagonal))
					diagonal = diagonal.add(length);
						
					// Determine the closest point
				if(Point.distance(point, horizontal) < Point.distance(point, vertical)) {
					if(Point.distance(point, horizontal) < Point.distance(point, diagonal))
						end = horizontal;
					else
						end = diagonal;
				} else {
					if(Point.distance(point, vertical) < Point.distance(point, diagonal))
						end = vertical;
					else
						end = diagonal;
				}
				
				return;
			}

				// Set for convinience rotation point
			x = (start.x + 0.5) * letterGrid.CELL_SPACE_WIDTH ;
			y = (start.y + 0.5) * letterGrid.CELL_SPACE_HEIGHT;
			
				// Draw selection
			graphics.clear();
			graphics.lineStyle(3, 0x990033, 1.0, true);
			graphics.drawRoundRect(-letterGrid.CELL_SPACE_WIDTH  / 2,
								   -letterGrid.CELL_SPACE_HEIGHT / 2,
								   (Point.distance(start, end) + 1) * letterGrid.CELL_SPACE_WIDTH,
								   letterGrid.CELL_SPACE_HEIGHT,
								   letterGrid.CELL_SPACE_WIDTH );
			
							// Rotate according to delta
			rotation = Math.atan2(delta.y, delta.x) * 180 / Math.PI;
		}
		
		private function outside(point:Point):Boolean
		{
			return !limitRect.containsPoint(point);
		}
		
		private function limitPoint(point:Point):Point
		{
			var newPoint:Point = point.clone();
			
				// Underflow
			if(point.x < limitRect.x)
				newPoint.x = limitRect.x;
			if(point.y < limitRect.y)
				newPoint.y = limitRect.y;

				// Overflow
			if(point.x > limitRect.right)
				newPoint.x = limitRect.right;
			if(point.y > limitRect.bottom)
				newPoint.y = limitRect.bottom;
				
			return newPoint;
		}
	}
}